_G.AtonementTracker = {}
local atonementTracker = _G.AtonementTracker
atonementTracker.SpellIDs = {
	["Atonement"] = {id = 194384},
	["Power Word: Shield"] = {id =17},
	["Penance"] = {id =47540},
	--["Mana Tea"] = {id =123761} 
}

function atonementTracker:GetLocalSpellNameFromID( id )
	if not id then
		return nil
	end
	local name, rank, icon, powerCost, isFunnel, powerType, castingTime, minRange, maxRange = GetSpellInfo( id )
	return name
end

function atonementTracker:GetSpellID( spell_name )
	if not spell_name then
		return nil
	end
	if not atonementTracker.SpellIDs[ spell_name ] then
		return nil
	end
	if not atonementTracker.SpellIDs[ spell_name ].name then
		-- Load the name, this should pick up the proper local name
		atonementTracker.SpellIDs[ spell_name ].name = atonementTracker:GetLocalSpellNameFromID( atonementTracker.SpellIDs[ spell_name ].id )
	end
	return atonementTracker.SpellIDs[ spell_name ].id
end

function atonementTracker:GetLocalSpellName( spell_name )
	if not spell_name then
		return nil
	end
	if not atonementTracker.SpellIDs[ spell_name ] then
		return nil
	end
	if not atonementTracker.SpellIDs[ spell_name ].name then
		-- Load the name, this should pick up the proper local name
		atonementTracker.SpellIDs[ spell_name ].name = atonementTracker:GetLocalSpellNameFromID( atonementTracker.SpellIDs[ spell_name ].id )
	end
	return atonementTracker.SpellIDs[ spell_name ].name
end
local myFrame = CreateFrame("frame", "AtonementTracker")

--Register Events
myFrame:RegisterEvent("PLAYER_LOGIN")
myFrame:RegisterEvent("ADDON_LOADED")
myFrame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
myFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

local myData = {
	player={},
	players={},
	statusBars = {},
	renewing_mist_targets = {},
	renewing_mist_heals = {},
	current_atonement_targets = 0,
	targets_under_80pct = 0
}
local Helpers = {}


-- Functions Section

local ordered_atonement_targets = {}
local sort_units_by_atonement_time = function( a, b )
	-- if not a or not a["Atonement"] or not a["Atonement"].expiration_time then
	-- 	return false
	-- else if not b or not b["Atonement"] or not b["Atonement"].expiration_time then
	-- 	return true
	-- end
	if AtonementTrackerDB and AtonementTrackerDB.sort and AtonementTrackerDB.sort == 'desc' then
		return a[atonementTracker:GetSpellID( "Atonement" )].expiration_time > b[atonementTracker:GetSpellID( "Atonement" )].expiration_time
	else
		return a[atonementTracker:GetSpellID( "Atonement" )].expiration_time < b[atonementTracker:GetSpellID( "Atonement" )].expiration_time
	end
end

function atonementTracker:updateStatusBars()
	myData.current_atonement_targets = 0
	myData.targets_under_80pct = 0
	myData.shortest_duration = 0
	myData.lowest_hp_pct = 100
	myData.avg_hp_pct = 100

	-- Update tooltip and grab spell damage from it
	SmiteTooltip:SetSpellByID(585)
	
	
	local smite_tooltip_text = SmiteTooltipTextLeft4:GetText()
	if not string.find(smite_tooltip_text, "Smites") then -- During GCD a line is added with Cooldown remaining
		smite_tooltip_text = SmiteTooltipTextLeft5:GetText()
	end
	local heal = tonumber(smite_tooltip_text:gsub(",",""):match("%d+")) / 2
	local totalHeal = 0
	
	
	-- Clean up our old ordered targets table
	for i = 1, #ordered_atonement_targets, 1 do
		table.remove( ordered_atonement_targets, 1 )
	end
	-- Count our current targets
	local total_hp_pct = 0
	for k,v in pairs(atonementTracker.data.units) do
		if v[atonementTracker:GetSpellID( "Atonement" )] and v[atonementTracker:GetSpellID( "Atonement" )].duration and v[atonementTracker:GetSpellID( "Atonement" )].duration > 0 and v[atonementTracker:GetSpellID( "Atonement" )].expiration_time > GetTime() then
			myData.current_atonement_targets = myData.current_atonement_targets + 1
			totalHeal = totalHeal + math.min(heal, v.max_hp - v.current_hp)
			if v.pct_hp then
				total_hp_pct = total_hp_pct + v.pct_hp
				if v.pct_hp < myData.lowest_hp_pct then
					myData.lowest_hp_pct = v.pct_hp
				end
				if v.pct_hp < 80 then
					myData.targets_under_80pct = myData.targets_under_80pct + 1
				end
			end
			table.insert( ordered_atonement_targets, v )
		end
	end

	myData.avg_hp_pct = ( total_hp_pct / #ordered_atonement_targets )
	-- Sort the ordered atonement targets array
	table.sort( ordered_atonement_targets, sort_units_by_atonement_time )
	
	-- Save our shortest duration, and set it on the comapct frame
	if AtonementTrackerDB and AtonementTrackerDB.mode == "compact" and #ordered_atonement_targets > 0 then
		atonementTracker.ui.compact_parent_frame.atonement_count_text:SetText( #ordered_atonement_targets )
		local remaining_time = ordered_atonement_targets[1][atonementTracker:GetSpellID( "Atonement" )].expiration_time - GetTime()
		if remaining_time < 0 then
			remaining_time = 0
		end
		atonementTracker.ui.compact_parent_frame.atonement_time_text:SetText(string.format("%4.1f", remaining_time) .. "s" )
		myData.shortest_duration = remaining_time
		
		if myData.lowest_hp_pct < 50 then
			atonementTracker.ui.compact_parent_frame.lowest_hp_text:SetTextColor(1 , 0, 0)
		else
			atonementTracker.ui.compact_parent_frame.lowest_hp_text:SetTextColor(0 , 1, 0)
		end
		atonementTracker.ui.compact_parent_frame.lowest_hp_text:SetText( math.floor( myData.lowest_hp_pct ) .. "%" )

		if myData.avg_hp_pct < 75 then
			atonementTracker.ui.compact_parent_frame.avg_hp_text:SetTextColor(1 , 0, 0)
		else
			atonementTracker.ui.compact_parent_frame.avg_hp_text:SetTextColor(0 , 1, 0)
		end
		atonementTracker.ui.compact_parent_frame.avg_hp_text:SetText( math.floor( myData.avg_hp_pct ) .. "%" )
		
		atonementTracker.ui.compact_parent_frame.avg_hp_icon:Show()
		atonementTracker.ui.compact_parent_frame.avg_hp_icon:SetDrawLayer("OVERLAY", 7)
		
		atonementTracker.ui.compact_parent_frame.smite_heal_text:SetText( Helpers:ReadableNumber(totalHeal, 1) )		
	else
		atonementTracker.ui.compact_parent_frame.avg_hp_icon:Hide()
		atonementTracker.ui.compact_parent_frame.atonement_count_text:SetText( "" )
		atonementTracker.ui.compact_parent_frame.atonement_time_text:SetText( "" )
		atonementTracker.ui.compact_parent_frame.lowest_hp_text:SetText( "" )
		atonementTracker.ui.compact_parent_frame.avg_hp_text:SetText( "" )
		atonementTracker.ui.compact_parent_frame.smite_heal_text:SetText( "" )
	end
	
	
	if #myData.statusBars < myData.current_atonement_targets then
		atonementTracker:createStatusBars( myData.current_atonement_targets - #myData.statusBars)
	end
	-- Hide and show the correct number of bars
	for i = 1, #myData.statusBars, 1 do
		if i <= myData.current_atonement_targets then
			myData.statusBars[i]:Show()
		else
			myData.statusBars[i]:Hide()
		end
	end
	
	-- Add new renewing mist targets
	local status_bar_index = 1
	for k,v in ipairs(ordered_atonement_targets) do
		local guid = v.guid
		--Set up the unit name on the bar.
		myData.statusBars[ status_bar_index ].playerName = v.name
		myData.statusBars[ status_bar_index ].value:SetText( v.name )
		local class_color = RAID_CLASS_COLORS[v.class_name]
		local remaining_time = v[atonementTracker:GetSpellID( "Atonement" )].expiration_time - GetTime()
		if remaining_time < 0 then
			remaining_time = 0
		end
		if class_color then
			myData.statusBars[ status_bar_index ].value:SetTextColor(class_color.r, class_color.g, class_color.b)
		end
		myData.statusBars[ status_bar_index ].value2:SetText(string.format("%4.1f", remaining_time) .. "s" )
		--Set the current health percentage
		if v.pct_hp then
			myData.statusBars[ status_bar_index ].health_pct:SetText( string.format("%4.1f", v.pct_hp ) .. "%" )
			local health_level = ( v.pct_hp - 35 )
			if health_level < 0 then
				health_level = 0
			elseif health_level > 65 then
				health_level = 65
			end
			local health_green = health_level / 65.0
			local health_red = 1 - health_green
			myData.statusBars[ status_bar_index ].health_pct:SetTextColor(health_red ,  health_green, 0)
		else
			myData.statusBars[ status_bar_index ].health_pct:SetText( "0.00%" )
		end
		if AtonementTrackerDB and AtonementTrackerDB.hide_health_pct then
			myData.statusBars[ status_bar_index ].health_pct:SetText( "" )
		end
		
		if v[atonementTracker:GetSpellID( "Atonement" )].last_heal then
			if v[atonementTracker:GetSpellID( "Atonement" )].last_heal.at > GetTime() - 1.2 then
				local heal_string = Helpers:ReadableNumber(v[atonementTracker:GetSpellID( "Atonement" )].last_heal.effective, 2) .. " ( O: ".. Helpers:ReadableNumber(v[atonementTracker:GetSpellID( "Atonement" )].last_heal.over, 2) .. " )"
				myData.statusBars[ status_bar_index ].heal_amt:SetText( heal_string )
				local fade = 1
				-- Are we fading in?  Fade in for .6 sec
				if v[atonementTracker:GetSpellID( "Atonement" )].last_heal.at > GetTime() - 0.6 then
					local delta_time = v[atonementTracker:GetSpellID( "Atonement" )].last_heal.at - ( GetTime() - 1.2 )
					fade = math.sin( ( delta_time / 0.6)  * ( math.pi / 2 ) )
				end
				myData.statusBars[ status_bar_index ].heal_amt:SetTextColor(0, 1, 0, fade)
			else
				myData.statusBars[ status_bar_index ].heal_amt:SetText( "" )
				myData.statusBars[ status_bar_index ].heal_amt:SetTextColor(0, 1, 0, 0)
			end
		end
		--Set the progressbar state
		myData.statusBars[ status_bar_index ]:SetMinMaxValues(0, v[atonementTracker:GetSpellID( "Atonement" )].duration)
		myData.statusBars[ status_bar_index ]:SetValue( remaining_time )
		-- Increment the status bar index for the next iteration
		status_bar_index = status_bar_index + 1
	end
end

function atonementTracker:createStatusBars( cnt )
	for i = 1, cnt, 1 do
		local yOffset = #myData.statusBars * 26 + 20
		-- Create the bar
	 	local bar = atonementTracker.ui:CreateProgressBar( "atonement_bar" .. #myData.statusBars, atonementTracker.ui.parent_frame )
		bar:SetPoint("TOPLEFT", 3, -3 - yOffset)
		bar:SetPoint("TOPRIGHT", -3, -3 - yOffset)
		bar:SetHeight(24)
		bar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
		bar:GetStatusBarTexture():SetHorizTile(false)
		bar:GetStatusBarTexture():SetVertTile(false)
		bar:SetStatusBarColor(1, 1, 0.444)
		bar:EnableMouse(true)

		bar.value = bar:CreateFontString(nil, "OVERLAY")
		bar.value:SetPoint("TOPLEFT", bar, "TOPLEFT", 4, 0)
		bar.value:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
		bar.value:SetJustifyH("LEFT")
		bar.value:SetShadowOffset(1, -1)
		bar.value:SetTextColor(0, 1, 0)
		-- Initialize the text with an empty string
		bar.value:SetText( "" )
		
		-- Do we have the player name for this bar?
		bar.value2 = bar:CreateFontString(nil, "OVERLAY")
		bar.value2:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", 4, 0)
		bar.value2:SetFont("Fonts\\FRIZQT__.TTF", 8, "OUTLINE")
		bar.value2:SetJustifyH("LEFT")
		bar.value2:SetShadowOffset(1, -1)
		bar.value2:SetTextColor(0, 1, 0)
		-- Initialize the text with an empty string
		bar.value2:SetText( "" )
		
		bar.health_pct = bar:CreateFontString(nil, "OVERLAY")
		bar.health_pct:SetPoint("RIGHT", bar, "RIGHT", 4, 0)
		bar.health_pct:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
		bar.health_pct:SetJustifyH("RIGHT")
		bar.health_pct:SetShadowOffset(1, -1)
		bar.health_pct:SetTextColor(1, 0, 0)
		-- Initialize the text with an empty string
		bar.health_pct:SetText( "" )
		
		bar.heal_amt = bar:CreateFontString(nil, "OVERLAY")
		bar.heal_amt:SetPoint("BOTTOM", bar, "BOTTOM", 0, 0)
		bar.heal_amt:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
		bar.heal_amt:SetJustifyH("CENTER")
		bar.heal_amt:SetShadowOffset(1, -1)
		bar.heal_amt:SetTextColor(0, 1, 0, 0)
		-- Initialize the text with an empty string
		bar.heal_amt:SetText( "" )
		
		-- Hide it so that we don't show empty bars.
		bar:Hide()
		-- Save it to our status bars table
		table.insert( myData.statusBars, bar )
	end
end

function atonementTracker:playerLogin()
	myFrame:SetScript("OnUpdate", atonementTracker.OnUpdate )
	-- Load our player into our data module
	atonementTracker.data:LoadUnitInfo("PLAYER")
	myData.player.guid = UnitGUID("PLAYER")
	myData.player.name = UnitName("PLAYER")
	myData.player.spec = GetSpecialization()
	-- Add ourselves to the database of seen players
	atonementTracker:CacheUserInfoForUnitID( "PLAYER" )
	
	atonementTracker:RegisterIndicators()
	
	DEFAULT_CHAT_FRAME:AddMessage( "Renewing Mist Tracker Loaded", 1, 1, 0.444 )
end


function atonementTracker:AnimateFrame( frame, elapsed )
	if frame.shouldHide and frame.shouldHide() then
		frame:Hide()
		return
	end
	atonementTracker:BlinkFrame( frame, elapsed )
	atonementTracker:BounceAnimateFrame(frame, elapsed )
end

function atonementTracker:BlinkFrame( frame, elapsed )
	if frame.shouldBlink and frame.shouldBlink() then
		local animation_time = frame.blinkAanimationTime or 0
		animation_time = animation_time + elapsed
		frame.blinkAanimationTime = animation_time
		-- Make sure the icon is visable if it is not on cooldown.
		local animation_step = math.sin( (math.fmod( animation_time, 1.5 ) / 1.5 ) * math.pi )
	
		frame:SetVertexColor(1, 1, animation_step);
	else
		frame:SetVertexColor(1, 1, 1);
	end
end

function atonementTracker:OnUpdate(elapsed)
	-- If we are not in healing spec exit this function
	if not atonementTracker:IsHealingSpec() then
			return
	end
	
	--Update our mana percentage
	local mana = UnitPower("PLAYER", SPELL_POWER_MANA)
	local max_mana = UnitPowerMax("Player",SPELL_POWER_MANA)
	
	if not max_mana or max_mana < 1 or not mana or mana < 1 then
		myData.player.mana_pct = 0
	else
		myData.player.mana_pct = (mana/max_mana) * 100
	end

	local members = GetNumGroupMembers()
	local grp_type = "party"
	if IsInRaid() then
		grp_type = "raid"
	end
	-- Check self
	atonementTracker.data:QuerySpellInfoForUnit( atonementTracker:GetSpellID( "Atonement" ), "PLAYER" )
	atonementTracker.data:UpdateUnitHealth( "PLAYER" )
	
	if members then
		for i = 1, members, 1 do
			local unit_id = grp_type .. i
			atonementTracker.data:LoadUnitInfo( unit_id )
			atonementTracker.data:QuerySpellInfoForUnit( atonementTracker:GetSpellID( "Atonement" ), unit_id )
			atonementTracker.data:UpdateUnitHealth( unit_id )
		end
	end
	atonementTracker:updateStatusBars()
	
	atonementTracker.ui:Animate( elapsed )
end

function atonementTracker:IsHealingSpec()
	return ( myData.player.spec == 1 )
end

function atonementTracker:QueryGlyphs()
	local glyphs = {}
	-- for i = 1, 6, 1 do
		-- local glyph_data = {}
		-- local enabled, glyphType, glyphTooltipIndex, glyphSpell, icon = GetGlyphSocketInfo(i)
		-- glyph_data.enabled = enabled
		-- glyph_data.glyph_type = glyphType
		-- glyph_data.tooltip_index = glyphTooltipIndex
		-- glyph_data.icon = icon
		-- glyph_data.spell_id = glyphSpell
		-- table.insert( glyphs, glyph_data )
	-- end
	myData.player.glyphs = glyphs
end

function atonementTracker:HasGlyph( spell_id )
	if not myData.player.glyphs then
		return false
	end
	if #myData.player.glyphs < 1 then
		return false
	end
	for k,v in pairs(myData.player.glyphs) do
		if v.spell_id and v.spell_id == spell_id then
			return true
		end
	end
	return false
end

--This will look nice in the code than atonementTracker:HasGlyph( 123763 ) 

function atonementTracker:HasManaTeaGlyph()
	return atonementTracker:HasGlyph( 123763 )
end

function atonementTracker:CacheUserInfoForUnitID( unit_id )
end

function atonementTracker:CombatLogEvent(...)
	-- If it is not from us, ignore it.
	if select(4,...) ~= myData.player.guid then
		return
	end
	if select(2,...) == "SPELL_PERIODIC_HEAL" then
		-- Change to spell id.
		if select(12,...) == atonementTracker:GetSpellID( "Atonement" ) then
		 	local seen_at = GetTime()
			local dest_guid = select(8,...)
			local amount = select(15,...)
			local over = select(16,...)
			local effective = amount - over
			atonementTracker.data:UpdateAtonementTick( seen_at, dest_guid, amount, over, effective)
		end
	end
end


function atonementTracker:RegisterIndicators()
	local true_fn = function() return true end
	local frame = atonementTracker.ui.parent_frame
	
	-- Create the texture
	local power_world_shield = atonementTracker.ui:CreateSpellTexture( frame, { "TOPLEFT", frame, "TOPLEFT", 34, 32 }, 32, 32, atonementTracker:GetSpellID( "Power Word: Shield" ) )
	local penance = atonementTracker.ui:CreateSpellTexture( frame, { "TOPLEFT", frame, "TOPLEFT", 68, 32 }, 32, 32, atonementTracker:GetSpellID( "Penance" ) )
	

	local power_world_shield_cast = CreateFrame("Button", nil, frame, "SecureActionButtonTemplate")
	power_world_shield_cast:SetPoint( "TOPLEFT", frame, "TOPLEFT", 34, 35 ) 
	power_world_shield_cast:SetSize(32, 35)
	power_world_shield_cast:SetAttribute( "type", "spell" )
	power_world_shield_cast:SetAttribute( "spell", atonementTracker:GetLocalSpellName( "Power Word: Shield" ) )
	
	local penance_cast = CreateFrame("Button", nil, frame, "SecureActionButtonTemplate")
	penance_cast:SetPoint( "TOPLEFT", frame, "TOPLEFT", 68, 35 ) 
	penance_cast:SetSize(32, 35)
	penance_cast:SetAttribute( "type", "spell" )
	penance_cast:SetAttribute( "spell", atonementTracker:GetLocalSpellName( "Penance" ) )
	

	--Setup all of the indicators to bounce
	atonementTracker.ui:Bounce( power_world_shield, 0.75, 7, true_fn )
	atonementTracker.ui:Bounce( penance, 0.75, 7, true_fn )
	
	-- Save the protected frames to the ui table
	if not atonementTracker.ui.protected_frames then
		atonementTracker.ui.protected_frames = {}
	end
	
	atonementTracker.ui.protected_frames.power_world_shield_cast = power_world_shield_cast
	atonementTracker.ui.protected_frames.penance_cast = penance_cast
	
	--Set up the grow for cooldowns
	atonementTracker.ui:SetCooldownGrow( power_world_shield  )
	atonementTracker.ui:SetCooldownGrow( penance  )

end

function OnEvent(self, event, ...)
	local arg1 = select(1, ...) or ""
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		atonementTracker:CombatLogEvent(...)
  elseif event == "PLAYER_LOGIN" then
		local localizedClass, englishClass = UnitClass("player")
		if englishClass ~= "PRIEST" then
			atonementTracker.ui:Hide()
			DEFAULT_CHAT_FRAME:AddMessage( "Atonement Tracker: This character is not a priest, not loading.", 1, 1, 0.444 )
			return
		end
		atonementTracker:playerLogin()
		atonementTracker:QueryGlyphs()

		-- If we are not in healing spec hide the frame and exit this function
		if not atonementTracker:IsHealingSpec() then
				atonementTracker.ui:Hide()
		else
			atonementTracker.ui:Show()
		end
	elseif event == "ADDON_LOADED" and arg1 == "atonement-tracker" then
		if not atonementTracker.ui_loaded then
			atonementTracker.ui:SetupBaseFrames()
			atonementTracker.ui:Hide()
			atonementTracker.ui_loaded = true

			if AtonementTrackerDB == nil then
		   	AtonementTrackerDB = {};
		  end
			if AtonementTrackerDB.scale then
				atonementTracker.ui:Scale( AtonementTrackerDB.scale )
			end
		end
	elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
		-- The spec number is passed to us, but this reads better.
		myData.player.spec = GetSpecialization()
		atonementTracker:QueryGlyphs()
		-- If we are not in healing spec hide the frame and exit this function
		if not atonementTracker:IsHealingSpec() then
				atonementTracker.ui:Hide()
		else
			atonementTracker.ui:Show()
		end
  end
end

myFrame:SetScript("OnEvent", OnEvent)

-- Helper functions

function Helpers:ReadableNumber(num, places)
    local ret
    local placeValue = ("%%.%df"):format(places or 0)
    if not num then
        return 0
    elseif num >= 1000000000000 then
        ret = placeValue:format(num / 1000000000000) .. " Tril" -- trillion
    elseif num >= 1000000000 then
        ret = placeValue:format(num / 1000000000) .. " Bil" -- billion
    elseif num >= 1000000 then
        ret = placeValue:format(num / 1000000) .. " Mil" -- million
    elseif num >= 1000 then
        ret = placeValue:format(num / 1000) .. "k" -- thousand
    else
        ret = num -- hundreds
    end
    return ret
end

